<?php

if ( ! class_exists( 'Emkp_Email_Alert_Manager' ) ):
    /**
     * Class Emkp_Email_Alert_Manager
     */
    class Emkp_Email_Alert_Manager implements \Zitec\MkpApiWrapper\AlertManager\AlertManagerInterface {
        /**
         * Logs file
         */
        const LOG_FILE = EMKP_ABSPATH . 'logs/alert_error_log.log';

        /**
         * @param string $type Possible values: curl_error, emag_error
         * @param string $message Error message
         * @param \Zitec\MkpApiWrapper\Requests\AbstractRequest $request Request object
         *
         * @return void
         */
        public function alert( $type, $message, \Zitec\MkpApiWrapper\Requests\AbstractRequest $request = null ) {
            $api_error_emails          = get_option( 'emkp_api_error_email' );
            $import_order_error_emails = get_option( 'emkp_import_order_email' );

            $api_error_emails          = explode( ',', $api_error_emails );
            $import_order_error_emails = explode( ',', $import_order_error_emails );

            $to            = null;
            $subject       = null;
            $email_message = null;

            switch ( $type ) {
                case 'curl_error':
                    $to            = $api_error_emails;
                    $subject       = __( 'Curl Error On Emag Marketplace API Call' );
                    $email_message = __( 'Curl Error: <strong>' . $message . '</strong>' );

                    break;

                case 'emag_error':
                    $to            = $import_order_error_emails;
                    $subject       = __( 'Emag Marketplace Response Error' );
                    $email_message = __( 'Emag Markplace Response: <strong>' . $message . '</strong>' );
                    break;
            }

            if ( ! is_null( $to ) && is_array( $to ) && ! empty( $to ) && ! is_null( $subject ) && ! is_null( $email_message ) ) {
                if ( ! is_null( $request ) ) {
                    $email_message .= "<br />Request type: <strong>" . get_class( $request ) . "</strong>";
                    $email_message .= "<br />Request data: " . print_r( $request->getRequestData(), true );
                }

                add_action( 'wp_mail_failed', [ $this, 'handle_wp_mail_failed' ] );

                try{
                wp_mail( $to, $subject, $email_message, [ 'Content-Type: text/html; charset=UTF-8' ] );
                } catch (WP_Error $e){
                    $this->handle_wp_mail_failed($e);
                }
            }
        }

        /**
         * @param WP_Error $error
         */
        public function handle_wp_mail_failed( WP_Error $error ) {
            $message = "Date: " . date( 'Y-m-d H:i:s' ) . "\n";
            $message .= "Code: " . $error->get_error_code() . "\n";
            $message .= "Message: " . $error->get_error_message() . "\n";
            $message .= "Error Data: " . print_r( $error->get_error_data(), true ) . "\n";
            $message .= str_repeat( '-', 100 );

            file_put_contents( self::LOG_FILE, $message, FILE_APPEND );
        }
    }
endif;
