<?php

if ( ! class_exists( 'Emkp_Awb_Handler' ) ) :
    class Emkp_Awb_Handler {
        public function init() {
            // display box only if it's an emag order
            if ( isset( $_GET['post'] ) && $this->get_emag_order_by_wc_id( (int) $_GET['post'] ) ) {
                // Create meta box
                add_action( 'add_meta_boxes', function () {
                    add_meta_box( 'emkp-awb-meta-box', __( 'eMAG AWB' ), [ $this, 'display_meta_box' ], 'shop_order' );
                } );
            }

            add_action( 'wp_ajax_nopriv_generate_emag_awb', function () {exit;} );
            add_action( 'wp_ajax_nopriv_download_emag_awb', function () {exit;} );

            // Add upload ajax endpoint
            add_action( 'wp_ajax_generate_emag_awb', [ $this, 'generate_awb_callback' ] );

            // Add awb download endpoint
            add_action( 'wp_ajax_download_emag_awb', [ $this, 'download_awb_callback' ] );
        }

        public function display_meta_box() {
            $order    = $this->get_emag_order_by_wc_id( (int) $_GET['post'] );
            $emagData = json_decode( $order['response'], true );

            $customerData  = array_key_exists( 'customer', $emagData ) ? $emagData['customer'] : [];
            $shipping_info = [
                'name'         => isset( $customerData['shipping_contact'] ) ? $customerData['shipping_contact'] : null,
                'contact'      => isset( $customerData['shipping_contact'] ) ? $customerData['shipping_contact'] : null,
                'phone1'       => isset( $customerData['shipping_phone'] ) ? $customerData['shipping_phone'] : null,
                'phone2'       => null,
                'locality_id'  => $customerData['shipping_locality_id'],
                'street'       => isset( $customerData['shipping_street'] ) ? $customerData['shipping_street'] : null,
                'postal_code'  => isset( $customerData['shipping_postal_code'] ) ? $customerData['shipping_postal_code'] : null,
                'legal_entity' => isset( $emagData['legal_entity'] ) ? $emagData['legal_entity'] : 0,
            ];

            $codAmount        = $this->get_cod_amount();
            $orderWeight      = $this->get_weight_of_delivery();
            $courier_accounts = $this->get_couriers();

            $locality_id       = $customerData['shipping_locality_id'];
            $selected_locality = null;
            if ( $locality_id ) {
                $selected_locality = ( new Emkp_Localities_Handler() )->get_locality( $locality_id );
            }

            $list = new Emkp_Awb_List();
            $list->prepare_items();
            $list->views();

            require_once EMKP_ABSPATH . 'templates/order/awb-meta-box.php';
        }

        public function generate_awb_callback() {
            global $wpdb;

            if ( ! isset( $_POST['emkp_awb'] ) || ! is_array( $_POST['emkp_awb'] ) || empty( $_POST['emkp_awb'] ) ) {
                echo json_encode( [ 'success' => false, 'message' => 'Missing AWB data!' ] );
                exit;
            }

            $postData    = $_POST['emkp_awb'];
            $postAwbData = $postData['awb_data'];

            if ( ! isset( $postData['awb_data'] ) || ! is_array( $postData['awb_data'] ) || empty( $postData['awb_data'] ) ) {
                echo json_encode( [ 'success' => false, 'message' => 'Missing AWB data!' ] );
                exit;
            }

            if ( ! isset( $postData['receiver'] ) || ! is_array( $postData['receiver'] ) || empty( $postData['receiver'] ) ) {
                echo json_encode( [ 'success' => false, 'message' => 'Missing receiver data!' ] );
                exit;
            }

            if ( ! isset( $postAwbData['courier_account_id'] ) || empty( $postAwbData['courier_account_id'] ) ) {
                echo json_encode( [ 'success' => false, 'message' => 'Select courier!' ] );
                exit;
            }

            if ( ! $this->courier_exists( $postAwbData['courier_account_id'] ) ) {
                echo json_encode( [ 'success' => false, 'message' => 'Invalid courier account!' ] );
                exit;
            }

            $order = $this->get_emag_order_by_wc_id( (int) $postData['order_id'] );

            if ( ! $order ) {
                echo json_encode( [ 'success' => false, 'message' => 'The order does not exist!' ] );
                exit;
            }

            try {
                $receiverData = $this->prepare_receiver_data( $postData['receiver'] );
            } catch ( \Exception $e ) {
                echo json_encode( [ 'success' => false, 'message' => $e->getMessage() ] );
                exit;
            }


            $awbData = [
                'order_id'           => (int) $order['emag_order_id'],
                'courier_account_id' => (int) $postAwbData['courier_account_id'],

                'insured_value'     => isset( $postAwbData['insured_value'] ) ? (float) $postAwbData['insured_value'] : null,
                'weight'            => isset( $postAwbData['weight'] ) ? (float) $postAwbData['weight'] : null,
                'envelope_number'   => isset( $postAwbData['envelope_number'] ) ? (int) $postAwbData['envelope_number'] : 0,
                'parcel_number'     => isset( $postAwbData['parcel_number'] ) ? (int) $postAwbData['parcel_number'] : 0,
                'observation'       => isset( $postAwbData['observation'] ) ? substr( $postAwbData['observation'], 0, 255 ) : null,
                'cod'               => isset( $postAwbData['cod'] ) ? $postAwbData['cod'] : null,
                'pickup_and_return' => (int) ( isset( $postAwbData['pickup_and_return'] ) && $postAwbData['pickup_and_return'] == 'on' ),
                'saturday_delivery' => (int) ( isset( $postAwbData['saturday_delivery'] ) && $postAwbData['saturday_delivery'] == 'on' ),
                'sameday_delivery'  => (int) ( isset( $postAwbData['sameday_delivery'] ) && $postAwbData['sameday_delivery'] == 'on' ),

                'sender'   => $this->prepare_sender_data($order),
                'receiver' => $receiverData,
            ];

            $awbData = array_filter( $awbData, function ( $row ) {
                return ! is_null( $row );
            } );

            $request = new \Zitec\MkpApiWrapper\Requests\Awb\SaveRequest( $awbData );

            $response = \Zitec\MkpApiWrapper\Client::sendRequest( $request );

            if ( $response->isError ) {
                echo json_encode( [ 'success' => false, 'message' => implode( "\n", $response->messages ) ] );
                exit;
            }

            if ( ! isset( $response->results->awb[0]->emag_id ) ) {
                echo json_encode( [ 'success' => false, 'message' => 'Error generating AWB!' ] );
                exit;
            }

            $wpdb->insert( $wpdb->prefix . 'emkp_awbs', [
                'emag_id'      => $response->results->awb[0]->emag_id,
                'order_id'     => $postData['order_id'],
                'courier_name' => $response->results->courier_name,
                'awb_number'   => $response->results->awb[0]->awb_number
            ] );

            echo json_encode( [ 'success' => true, 'message' => 'AWB generated successfully!' ] );
            exit;
        }

        public function download_awb_callback() {
            global $wpdb;

            if ( ! isset( $_POST['awb_id'] ) ) {
                echo json_encode( [ 'success' => false, 'message' => 'Missing AWB id!' ] );

                exit;
            }

            if ( ! isset( $_POST['format'] ) ) {
                echo json_encode( [ 'success' => false, 'message' => 'Select AWB format' ] );

                exit;
            }

            $format = strtoupper( esc_html( $_POST['format'] ) );

            if ( ! in_array( $format, [ 'A4', 'A5', 'A6' ] ) ) {
                echo json_encode( [ 'success' => false, 'message' => 'Invalid AWB format' ] );

                exit;
            }

            $awb = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}emkp_awbs WHERE emag_id = %d LIMIT 1", (int) $_POST['awb_id'] ) );

            if ( ! $awb ) {
                echo json_encode( [ 'success' => false, 'message' => 'Invalid AWB!' ] );

                exit;
            }

            $wp_upload_dir = wp_get_upload_dir();

            $awb_directory_name = 'emag_awb';
            $awb_base_path      = $wp_upload_dir['basedir'] . '/' . $awb_directory_name;
            $awb_file_name      = 'awb_' . $awb->emag_id . '_' . $format . '.pdf';
            $awb_file_path      = $awb_base_path . '/' . $awb_file_name;

            if ( ! file_exists( $awb_base_path ) && ( ! @mkdir( $awb_base_path ) && ! @is_dir( $awb_base_path ) ) ) {
                echo json_encode( [ 'success' => false, 'message' => 'Cannot download AWB. Check directory permission!' ] );

                exit;
            }

            if ( ! file_exists( $awb_file_path ) ) {
                try {
                    $emag_awb = \Zitec\MkpApiWrapper\Client::sendRequest( new \Zitec\MkpApiWrapper\Requests\Awb\ReadRequest( [
                        'emag_id' => $awb->emag_id,
                        'format'  => $format,
                    ] ) );

                    file_put_contents( $awb_file_path, $emag_awb );
                } catch ( \Zitec\MkpApiWrapper\Exceptions\FailedRequestException $exception ) {
                    echo json_encode( [ 'success' => false, 'message' => 'Invalid request! Please check API configuration' ] );

                    exit;
                }
            }

            echo json_encode( [ 'success' => true, 'url' => $wp_upload_dir['baseurl'] . '/' . $awb_directory_name . '/' . $awb_file_name ] );

            exit;
        }

        /**
         * @param array $receiver_data
         *
         * @return array
         * @throws Exception
         */
        private function prepare_receiver_data( array $receiver_data ) {
            $data = [
                'name'         => null,
                'contact'      => null,
                'phone1'       => null,
                'phone2'       => null,
                'legal_entity' => null,
                'locality_id'  => null,
                'street'       => null,
                'zipcode'      => null,
            ];

            $required_fields = [
                'name'        => 'Name',
                'contact'     => 'Contact Person',
                'phone1'      => 'Phone Number 1',
                'locality_id' => 'Locality',
                'street'      => 'Street',
            ];

            // Validate required fields
            foreach ( $required_fields as $field => $label ) {
                if ( ! isset( $receiver_data[ $field ] ) || empty( $receiver_data[ $field ] ) ) {
                    throw new \Exception( 'Missing field ' . $label );
                }
            }

            // Prepare data
            foreach ( array_keys( $data ) as $field ) {
                if ( isset( $receiver_data[ $field ] ) && ! empty( $receiver_data[ $field ] ) ) {
                    $data[ $field ] = $receiver_data[ $field ];
                }
            }

            $data['legal_entity'] = (int) $data['legal_entity'];

            return array_filter( $data );
        }

        /**
         * @return array
         */
        private function prepare_sender_data( array $order ) {
            $fields = [ 'phone1', 'phone2', 'locality_id', 'street' ];

            $data = [];
            foreach ( $fields as $field ) {
                $data[ $field ] = get_option( 'emkp_shipping_' . $field );
            }

            $data['contact'] = $data['zipcode'] = 'AUTO-GENERATED';

            $emagOrderData = json_decode($order['response'], true);
            if(isset($emagOrderData['vendor_name'])) {
                $data['name'] = $emagOrderData['vendor_name'];
            }

            return array_filter( $data );
        }

        /**
         * @return float|null
         */
        private function get_cod_amount() {
            $order = wc_get_order( (int) $_GET['post'] );

            if ( get_option( 'emkp_payment_cod' ) == $order->get_payment_method() ) {
                return $order->get_total();
            }

            return null;
        }

        /**
         * @return float|int
         */
        private function get_weight_of_delivery() {
            $order = wc_get_order( (int) $_GET['post'] );

            $totalWeight = 0;
            /** @var WC_Order_item $item */
            foreach ( $order->get_items() as $item ) {
                $product = wc_get_product( $item['get_weight_of_delivery'] );
                if ( $product ) {
                    $totalWeight += $product->get_weight();
                }
            }

            return $totalWeight;
        }

        /**
         * @param $order_id
         *
         * @return array|null|object
         */
        private function get_emag_order_by_wc_id( $order_id ) {
            global $wpdb;

            $order = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}emkp_orders_queue WHERE wc_order_id = %d LIMIT 1 ", [ $order_id ] ), ARRAY_A );

            if ( $order ) {
                return $order;
            }

            return null;
        }

        /**
         * @return array
         */
        private function get_localities() {
            return ( new Emkp_Localities_Handler() )->get_localities();
        }

        /**
         * @return array
         */
        private function get_couriers() {
            global $wpdb;

            $db_couriers = $wpdb->get_results( "SELECT emag_id, name, display_name FROM {$wpdb->prefix}emkp_couriers ORDER BY display_name ASC", ARRAY_A );

            $couriers = [];
            foreach ( $db_couriers as $courier ) {
                $couriers[ $courier['emag_id'] ] = $courier['display_name'] . ' (' . $courier['name'] . ')';
            }

            return $couriers;
        }

        /**
         * @param $courier_id
         *
         * @return bool
         */
        private function courier_exists( $courier_id ) {
            global $wpdb;

            $result = $wpdb->get_row( $wpdb->prepare( "SELECT COUNT(id) AS existing FROM {$wpdb->prefix}emkp_couriers WHERE emag_id = %d", $courier_id ) );

            return (bool) $result->existing;
        }
    }

    ( new Emkp_Awb_Handler )->init();
endif;
