<?php

if ( ! class_exists( 'Emkp_Invoice_Handler' ) ) :
    class Emkp_Invoice_Handler {
        /**
         * @var wpdb
         */
        private $wpdb;

        public function init() {
            global $wpdb;

            $this->wpdb = $wpdb;

            // Add upload ajax endpoint
            add_action( 'wp_ajax_nopriv_upload_emag_invoice', function () {exit;} );
            add_action( 'wp_ajax_upload_emag_invoice', [ $this, 'upload_invoice_callback' ] );

            // display box only if it's an emag order
            if ( ! isset( $_GET['post'] ) || ( isset( $_GET['post'] ) && ! $this->get_emag_order_id_by_wc_id( (int) $_GET['post'] ) ) ) {
                return;
            }

            // Create meta box
            add_action( 'add_meta_boxes', function () {
                add_meta_box( 'emkp-invoice-meta-box', __( 'eMAG Invoice' ), [ $this, 'display_meta_box' ], 'shop_order', 'side' );
            } );
        }


        public function display_meta_box() {
            $invoice_url = $this->get_invoice_url( (int) $_GET['post'] );

            require_once EMKP_ABSPATH . 'templates/order/invoices-meta-box.php';
        }

        public function upload_invoice_callback() {
            if ( empty( $_FILES ) ) {
                echo json_encode( [ 'success' => false, 'message' => __( 'No file selected!' ) ] );
                exit;
            }

            // validate file format
            if ( finfo_file( finfo_open( FILEINFO_MIME_TYPE ), $_FILES[0]['tmp_name'] ) !== 'application/pdf' ) {
                echo json_encode( [ 'success' => false, 'message' => __( 'Invalid file selected!' ) ] );
                exit;
            }

            // validate order
            if ( ! isset( $_POST['order_id'] ) || ! $this->get_emag_order_id_by_wc_id( (int) $_POST['order_id'] ) ) {
                echo json_encode( [ 'success' => false, 'message' => __( 'Invalid order!' ) ] );
                exit;
            }

            $uploaded_file = wp_handle_upload( $_FILES[0], [ 'test_form' => false ] );

            $response = [
                'message' => null,
            ];

            if ( $uploaded_file && ! isset( $uploaded_file['error'] ) && file_exists( $uploaded_file['file'] ) ) {
                $attachment_data = [
                    'order_id'       => $this->get_emag_order_id_by_wc_id( (int) $_POST['order_id'] ),
                    'url'            => $uploaded_file['url'],
                    'force_download' => 1,
                ];

                try {
                    // send request to emag
                    $emag_response = \Zitec\MkpApiWrapper\Client::sendRequest( new \Zitec\MkpApiWrapper\Requests\Order\AttachmentsRequest( [ $attachment_data ], true ) );
                } catch ( \Zitec\MkpApiWrapper\Exceptions\FailedRequestException $exception ) {
                    $emag_response = (object) [
                        'isError'  => true,
                        'messages' => [
                            'Invalid request! Please check API configuration',
                        ]
                    ];
                }

                if ( ! $emag_response->isError ) {
                    $response['success'] = true;
                    $response['message'] = 'The invoice has been sent to eMAG!';

                    $query = $this->wpdb->prepare( "INSERT INTO {$this->wpdb->prefix}emkp_orders_invoices (order_id, file_name, url) VALUES(%d, %s, %s)", (int) $_POST['order_id'],
                        $uploaded_file['file'], $uploaded_file['url'] );

                    $this->wpdb->query( $query );
                } else {
                    $response['success'] = false;
                    $response['message'] = 'ERROR: ' . implode( '; ', $emag_response->messages );

                    unlink( $uploaded_file['file'] );
                }
            } else {
                $response['success'] = false;
                $response['message'] = $uploaded_file['error'];
                if ( file_exists( $uploaded_file['file'] ) ) {
                    unlink( $uploaded_file['file'] );
                }
            }

            echo json_encode( $response );
            exit;
        }

        /**
         * Get Emkp order id using WooCommerce order id
         *
         * @param int $order_id
         *
         * @return mixed
         */
        private function get_emag_order_id_by_wc_id( $order_id ) {
            $order = $this->wpdb->get_row( $this->wpdb->prepare( "SELECT emag_order_id FROM {$this->wpdb->prefix}emkp_orders_queue WHERE wc_order_id = %d LIMIT 1 ", $order_id ) );

            return ! is_null( $order ) ? $order->emag_order_id : false;
        }

        /**
         * @param $order_id
         *
         * @return null
         */
        private function get_invoice_url( $order_id ) {
            $invoice = $this->wpdb->get_row( $this->wpdb->prepare( "SELECT url FROM {$this->wpdb->prefix}emkp_orders_invoices WHERE order_id = %d LIMIT 1", $order_id ) );

            if ( ! $invoice ) {
                return null;
            }

            return $invoice->url;
        }
    }

    ( new Emkp_Invoice_Handler() )->init();
endif;
