<?php

if ( ! class_exists( 'Emkp_Localities_Handler' ) ):
    class Emkp_Localities_Handler {

        public function init() {
            add_action( 'wp_ajax_nopriv_emkp_import_localities', function () {
                exit;
            } );

            // Add upload ajax endpoint
            add_action( 'wp_ajax_emkp_import_localities', [ $this, 'import_shipping_localities' ] );

            add_action( 'wp_ajax_emkp_localities_search', [ $this, 'emkp_localities_search' ] );
        }

        public function import_shipping_localities() {
            /**
             * @var wpdb $wpdb
             */
            global $wpdb;

            $emag_localities = [];
            $page            = 1;
            try {
                do {
                    $request = new Zitec\MkpApiWrapper\Requests\Locality\ReadRequest( [
                        'currentPage' => $page,
                        'itemsPerPage' => 500,
                    ] );

                    $response = Zitec\MkpApiWrapper\Client::sendRequest( $request );

                    $emag_localities = array_merge( $emag_localities, $response->results );

                    $page ++;
                } while ( isset( $response->results ) && count( $response->results ) );
            } catch ( \Zitec\MkpApiWrapper\Exceptions\FailedRequestException $exception ) {
                echo json_encode( [ 'success' => false, 'message' => $exception->getMessage() ] );
                exit;
            }

            if ( empty( $emag_localities ) ) {
                echo json_encode( [ 'success' => false, 'message' => __( 'No localities retrieved!' ) ] );
                exit;
            }

            $existing_localities = $wpdb->get_col( "SELECT emag_id FROM {$wpdb->prefix}emkp_localities" );

            $localities = [];

            foreach ( $emag_localities as $locality ) {
                if ( ! is_object( $locality ) || ! property_exists( $locality, 'emag_id' ) || ! property_exists( $locality, 'name' ) || empty( $locality->name ) ) {
                    continue;
                }

                $localities[ $locality->emag_id ] = '(' . implode( ',', [
                        $locality->emag_id,
                        "'" . $locality->name_latin . "'",
                        "'" . $locality->region2_latin . "'",
                    ] ) . ')';
            }

            $deleted_localities = array_diff( $existing_localities, array_keys( $localities ) );

            if ( ! empty( $localities ) ) {
                $vals = array_chunk( $localities, 300 );

                foreach ( $vals as $val ) {
                    $wpdb->query( "INSERT INTO {$wpdb->prefix}emkp_localities (emag_id, name, region) VALUES " . implode( ',', $val ) . "
                    ON DUPLICATE KEY UPDATE name = VALUES(name), region = VALUES(region)" );
                }
            }

            if ( ! empty( $deleted_localities ) ) {
                $wpdb->query( "DELETE FROM {$wpdb->prefix}emkp_localities WHERE id IN (" . implode( ',', $deleted_localities ) . ")" );
            }

            update_option( 'emkp_localities_imported', true );
            echo json_encode( [ 'success' => true, 'message' => __( 'The localities have been imported!' ) ] );
            exit;
        }

        public function emkp_localities_search() {
            $search_query = isset( $_GET['search'] ) ? $_GET['search'] : null;

            $limit = $search_query ? null : 100;

            echo json_encode( $this->get_localities( wp_strip_all_tags( $search_query ), $limit ) );
            exit;
        }

        /**
         * @param null $search_query
         * @param null $limit
         *
         * @return array
         */
        public function get_localities( $search_query = null, $limit = null ) {
            global $wpdb;

            $query = "SELECT `emag_id`, `name`, `region` FROM {$wpdb->prefix}emkp_localities";

            if ( $search_query ) {
                $query .= " WHERE `name` LIKE '%{$wpdb->esc_like($search_query)}%' ";
            }

            $query .= ' ORDER BY name ASC';

            if ( $limit ) {
                $query .= ' LIMIT ' . $limit;
            }

            $db_localities = $wpdb->get_results( $query, ARRAY_A );

            $localities = [];

            foreach ( $db_localities as $locality ) {
                $localities[ $locality['emag_id'] ] = $locality['name'] . ' (' . $locality['region'] . ')';
            }

            return $localities;
        }

        public function get_locality( $locality_id ) {
            global $wpdb;

            return $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}emkp_localities WHERE emag_id = %d", $locality_id ), ARRAY_A );
        }

    }

    ( new Emkp_Localities_Handler() )->init();
endif;
