<?php

if ( ! class_exists( 'Emkp_Product_Queue' ) ):
    /**
     * Class Emkp_Product_Queue
     */
    class Emkp_Product_Queue {
        /**
         * @var null|wpdb
         */
        private $wpdb = null;

        /**
         * @var array
         */
        private $products_queue = [];

        public function __construct() {
            global $wpdb;

            $this->add_hooks();

            $this->wpdb = $wpdb;
        }

        private function add_hooks() {
            add_action( 'save_post_product', [ $this, 'save_post_product_callback' ], 10, 5 );
            add_action( 'woocommerce_product_set_stock', [ $this, 'woocommerce_product_set_stock_callback' ], 10, 5 );
            add_action( 'woocommerce_checkout_order_processed', [ $this, 'woocommerce_new_order_callback' ], 10, 5 );
            add_action( 'wp_trash_post', [ $this, 'wp_trash_post_callback' ], 10, 5 );
            add_action( 'delete_post', [ $this, 'delete_post_callback' ], 10, 5 );
            add_action( 'emkp_product_category_deleted', [ $this, 'emkp_product_category_deleted_callback' ], 10, 5 );
            add_action( 'untrash_post', [ $this, 'restore_product_callback' ], 10, 5 );
            add_action( 'woocommerce_product_set_stock', [
                $this,
                'woocommerce_increase_reduce_stock_callback'
            ], 10, 5 );
            add_action( 'woocommerce_product_reduce_stock', [
                $this,
                'woocommerce_increase_reduce_stock_callback'
            ], 10, 5 );
        }

        /**
         * @param int $post_id
         */
        public function save_post_product_callback( $post_id ) {
            $product = wc_get_product( $post_id );

            if ( ! ( $product instanceof WC_Product ) ) {
                return;
            }

            if ( get_post_meta( $post_id, 'untrash_action', true ) === "yes" ) {
                delete_post_meta( $post_id, 'untrash_action' );

                return;
            }

            if ( ! ( array_key_exists( '_emkp_visible_to_marketplace', $_POST ) && $_POST['_emkp_visible_to_marketplace'] == 'yes' ) && ( get_post_meta( $post_id, 'emkp_sent_to_emag', true ) == "yes" ) ) {
                $this->add_to_list( $post_id, 'deleted' );
            } elseif ( ! array_key_exists( '_emkp_visible_to_marketplace', $_POST ) && $product->get_meta( '_emkp_visible_to_marketplace' ) == "yes" ) {
                $this->add_to_list( $post_id, 'deleted' );
            } else {
                $this->add_to_queue( $product, false, null, true );
            }

            $this->save_queue();
        }

        /**
         * @param int $order_id
         */
        public function woocommerce_new_order_callback( $order_id ) {
            $order       = wc_get_order( $order_id );
            $order_items = $order->get_items();

            /** @var WC_Order_Item_Product $order_item */
            foreach ( $order_items as $order_item ) {
                if ( ! ( $order_item instanceof WC_Product ) ) {
                    return;
                }

                $this->add_to_queue( $order_item->get_product(), true, $order_item );
            }

            $this->save_queue();
        }

        /**
         * @param $product
         */
        public function woocommerce_product_set_stock_callback( $product ) {
            if ( ! ( $product instanceof WC_Product ) ) {
                return;
            }

            $this->add_to_queue( $product );

            $this->save_queue();
        }

        /**
         * @param $product
         */
        public function woocommerce_increase_reduce_stock_callback( $product ) {
            if ( ! ( $product instanceof WC_Product ) ) {
                return;
            }

            if ( get_post_meta( $product->get_id(), 'emkp_sent_to_emag', true ) == "yes" ) {
                $this->add_to_list( $product->get_id(), 'updated' );
                $this->save_queue();
            }
        }

        /**
         * @param integer $product_id
         */
        public function wp_trash_post_callback( $product_id ) {
            $product = wc_get_product( $product_id );

            if ( ! ( $product instanceof WC_Product ) || $product instanceof WC_Product_Variation ) {
                return;
            }

            if ( get_post_meta( $product_id, 'emkp_sent_to_emag', true ) === "yes" ) {
                $this->add_to_list( $product_id, 'deleted' );
                $this->save_queue();
            }
        }


        public function delete_post_callback( $product_id ) {
            $product = wc_get_product( $product_id );

            if ( ! ( $product instanceof WC_Product_Variation ) ) {
                return;
            }

            $product = wc_get_product( $product->get_parent_id() );

            if ( $product && $product->get_meta( '_emkp_visible_to_marketplace' ) === 'yes' ) {
                $this->add_to_list( $product_id, 'deleted' );
                $this->save_queue();
            }
        }

        /**
         * @param $product_id
         */
        public function restore_product_callback( $product_id ) {
            $product = wc_get_product( $product_id );

            if ( ! ( $product instanceof WC_Product ) || $product instanceof WC_Product_Variation ) {
                return;
            }

            update_post_meta( $product_id, 'untrash_action', 'yes' );

            $this->add_to_list( $product_id, 'updated' );
            $this->save_queue();
        }

        /**
         * @param WP_Post[] $products
         */
        public function emkp_product_category_deleted_callback( $products ) {
            foreach ( $products as $product ) {
                if ( get_post_meta( $product->ID, '_emkp_visible_to_marketplace', true ) != "yes" ) {
                    return;
                }

                $this->add_to_list( $product->ID, 'deleted' );
            }

            $this->save_queue();
        }

        /**
         * @param WC_Product $product
         * @param bool $from_order
         * @param WC_Order_Item_Product|null $order_item
         * @param bool $is_post
         */
        private function add_to_queue( WC_Product $product, $from_order = false, WC_Order_Item_Product $order_item = null, $is_post = false ) {
            if ( ! $product instanceof WC_Product_Simple && ! $product instanceof WC_Product_Variable ) {
                return;
            }

            if ( ! $is_post ) {
                $is_visible  = $product->get_meta( '_emkp_visible_to_marketplace' ) == 'yes';
                $valid_stock = $product->get_manage_stock();
                $sku         = $product->get_sku();
            } else {
                $is_visible  = array_key_exists( '_emkp_visible_to_marketplace', $_POST ) && $_POST['_emkp_visible_to_marketplace'] == 'yes';
                $valid_stock = array_key_exists( '_manage_stock', $_POST ) && $_POST['_manage_stock'] == 'yes';
                $sku         = esc_html( array_key_exists( '_sku', $_POST ) ? $_POST['_sku'] : null );
            }

            if ( $is_visible && $valid_stock && strlen( $sku ) ) {
                $this->add_to_list( $product->get_id(), 'updated' );
            }
        }

        /**
         * @param int $product_id
         * @param string $state
         */
        private function add_to_list( $product_id, $state ) {
            $key = $product_id . '_' . $state;

            if ( ! array_key_exists( $key, $this->products_queue ) ) {
                $this->products_queue[ $key ] = [
                    'product_id' => $product_id,
                    'state'      => $state,
                ];
            }
        }

        private function save_queue() {
            if ( empty( $this->products_queue ) ) {
                return;
            }

            $this->wpdb->query( 'START TRANSACTION' );

            $success = true;
            foreach ( $this->products_queue as $product ) {
                $result = $this->wpdb->query( $this->wpdb->prepare( "INSERT INTO {$this->wpdb->prefix}emkp_products_queue (product_id, state) VALUES (%d, %s)", $product ) );

                $success = $success && is_numeric( $result );
            }

            $this->wpdb->query( $success ? 'COMMIT' : 'ROLLBACK' );
        }
    }

    new Emkp_Product_Queue();
endif;
