<?php

if ( ! class_exists( 'Emkp_Product_Tab' ) ):
    /**
     * Class Emkp_Product_Tab
     */
    class Emkp_Product_Tab {
        public function __construct() {
            // add tab
            add_filter( 'woocommerce_product_data_tabs', [ $this, 'woocommerce_product_data_tabs_callback' ], 99, 1 );

            // add tab content
            add_action( 'woocommerce_product_data_panels', [ $this, 'woocommerce_product_data_panels_callback' ] );

            // save tab content
            add_action( 'woocommerce_process_product_meta', [ $this, 'woocommerce_process_product_meta_callback' ] );

            // add tab to variations
            add_action( 'woocommerce_product_after_variable_attributes', [
                $this,
                'woocommerce_product_after_variable_attributes_callback'
            ], 10, 3 );

            // save variation fields
            add_action( 'woocommerce_save_product_variation', [
                $this,
                'woocommerce_save_product_variation_callback'
            ], 10, 2 );
        }

        public function woocommerce_product_data_tabs_callback( array $product_data_tabs ) {
            $product_data_tabs['emkp-tab'] = array(
                'label'  => __( 'Emag Marketplace' ),
                'target' => 'emkp-tab',
            );

            return $product_data_tabs;
        }

        public function woocommerce_product_data_panels_callback() {
            global $woocommerce, $post;

            require_once( EMKP_ABSPATH . 'templates/product/emag-tab.php' );
        }

        public function woocommerce_process_product_meta_callback( $post_id ) {
            $fields = [
                '_emkp_visible_to_marketplace' => 'checkbox',
                '_emkp_pnk'                    => 'text',
                '_emkp_brand'                  => 'text',
                '_emkp_warranty'               => 'text',
            ];

            foreach ( $fields as $field => $type ) {
                if ( $type == 'checkbox' ) {
                    $value = array_key_exists( $field, $_POST ) ? 'yes' : 'no';
                } else {
                    $value = array_key_exists( $field, $_POST ) ? esc_html( $_POST[ $field ] ) : null;
                }

                update_post_meta( $post_id, $field, $value );
            }
        }

        public function woocommerce_product_after_variable_attributes_callback( $loop, $variation_data, $variation ) {
            woocommerce_wp_checkbox( array(
                'id'          => '_emkp_visible_to_marketplace[' . $variation->ID . ']',
                'label'       => __( 'Visible To Marketplace' ),
                'description' => __( 'Is variation visible to Emag Marketplace?' ),
                'default'     => '0',
                'desc_tip'    => false,
                'value'       => get_post_meta( $variation->ID, '_emkp_visible_to_marketplace', true ),
            ) );

            woocommerce_wp_text_input( [
                'id'          => '_emkp_pnk[' . $variation->ID . ']',
                'label'       => __( 'Emag Part Number Key' ),
                'description' => null,
                'default'     => null,
                'desc_tip'    => false,
                'style'       => 'max-width:250px',
                'value'       => get_post_meta( $variation->ID, '_emkp_pnk', true ),
            ] );
        }

        public function woocommerce_save_product_variation_callback( $post_id ) {
            // visible to marketplace
            $visible = isset( $_POST['_emkp_visible_to_marketplace'][ $post_id ] ) ? 'yes' : 'no';
            update_post_meta( $post_id, '_emkp_visible_to_marketplace', $visible );

            // pnk
            $pnk = array_key_exists( '_emkp_pnk', $_POST ) && array_key_exists( $post_id, $_POST['_emkp_pnk'] ) ? esc_html( $_POST['_emkp_pnk'][ $post_id ] ) : null;
            update_post_meta( $post_id, '_emkp_pnk', esc_attr( $pnk ) );
        }
    }

    new Emkp_Product_Tab();
endif;
