<?php

if ( ! class_exists( 'Emkp_Schema_Updater' ) ) :

    /**
     * Class Emkp_Schema_Updater
     */
    class Emkp_Schema_Updater {
        /**
         * @var array
         */
        public static $tablenames = [];

        /**
         * @var array
         */
        public static $config_options = [
            'emkp_api_url',
            'emkp_username',
            'emkp_password',
            'emkp_locale',
            'emkp_currency',
            'emkp_api_error_email',
            'emkp_product_queue_limit',
            'emkp_vat_rate',
            'emkp_min_sale_price',
            'emkp_max_sale_price',
            'emkp_import_order_email',
            'emkp_initial_order_status',
            'emkp_finalized_order_status',
            'emkp_cancelled_order_status',
            'emkp_handling_time',

            'emkp_shipping_phone1',
            'emkp_shipping_phone2',
            'emkp_shipping_locality_id',
            'emkp_shipping_street',
            'emkp_payment_cod',
            'emkp_localities_imported',
            'emkp_payment_cod',
            'emkp_payment_bank_transfer',
            'emkp_payment_card',

            'emkp_migration_version',

        ];

        /**
         * @var wpdb
         */
        private $wpdb;

        /**
         * Emkp_Schema_Updater constructor.
         */
        public function __construct() {
            global $wpdb;
            $this->wpdb = $wpdb;

            self::$tablenames = [
                'EMKP_CATEGORIES'                       => $this->wpdb->prefix . 'emkp_categories',
                'EMKP_CHARACTERISTICS'                  => $this->wpdb->prefix . 'emkp_characteristics',
                'EMKP_CATEGORIES_CHARACTERISTICS'       => $this->wpdb->prefix . 'emkp_categories_characteristics',
                'EMKP_CATEGORIES_MATCH'                 => $this->wpdb->prefix . 'emkp_categories_match',
                'EMKP_MATCH_CHARACTERISTICS_ATTRIBUTES' => $this->wpdb->prefix . 'emkp_match_characteristics_attributes',
                'EMKP_VAT_RATES'                        => $this->wpdb->prefix . 'emkp_vat_rates',
                'EMKP_PRODUCTS_QUEUE'                   => $this->wpdb->prefix . 'emkp_products_queue',
                'EMKP_ORDER_IDS'                        => $this->wpdb->prefix . 'emkp_orders_queue',
                'EMKP_ORDERS_INVOICES'                  => $this->wpdb->prefix . 'emkp_orders_invoices',
                'EMKP_LOCALITIES'                       => $this->wpdb->prefix . 'emkp_localities',
                'EMKP_COURIERS'                         => $this->wpdb->prefix . 'emkp_couriers',
                'EMKP_AWBS'                             => $this->wpdb->prefix . 'emkp_awbs',
            ];
        }

        /**
         * Get database table schema
         */
        public function install_schema() {
            $this->wpdb->hide_errors();
            require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

            $collate = '';

            if ( $this->wpdb->has_cap( 'collation' ) ) {
                $collate = $this->wpdb->get_charset_collate();
            }

            $tablenames = self::$tablenames;
            $sql        = ( "
        CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_CATEGORIES']}
                (id mediumint(9) unsigned NOT NULL auto_increment,
                emag_category_id INT NOT NULL UNIQUE,
                emag_parent_id INT DEFAULT NULL,
                name VARCHAR(255) NOT NULL,
                PRIMARY KEY (id)
                ) $collate;
                " );
            dbDelta( $sql );

            $sql = ( "
        CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_CHARACTERISTICS']}
                (id mediumint(9) unsigned NOT NULL auto_increment,
                emag_characteristic_id INT NOT NULL UNIQUE,
                name VARCHAR(255) NOT NULL,
                type_id INT NOT NULL,
                display_order INT NOT NULL,
                allow_new_value TINYINT(1) NOT NULL,
                PRIMARY KEY (id)
                ) $collate;
                " );
            dbDelta( $sql );

            $sql = ( "
        CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_CATEGORIES_CHARACTERISTICS']}
                (id mediumint(9) unsigned NOT NULL auto_increment,
                emag_category_id INT NOT NULL,
                emag_characteristic_id INT NOT NULL,
                is_mandatory TINYINT(1) NOT NULL,
                PRIMARY KEY (id),
                CONSTRAINT fk_emkp_categories_category_id FOREIGN KEY (emag_category_id) 
                REFERENCES {$tablenames['EMKP_CATEGORIES']}(emag_category_id) ON DELETE CASCADE,
                CONSTRAINT fk_emkp_categories_characteristic_id FOREIGN KEY (emag_characteristic_id) 
                REFERENCES {$tablenames['EMKP_CHARACTERISTICS']}(emag_characteristic_id) ON DELETE CASCADE
                ) $collate;
                " );
            dbDelta( $sql );

            $sql = ( "
        CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_CATEGORIES_MATCH']}
                (id mediumint(9) unsigned NOT NULL auto_increment,
                emag_category_id INT NOT NULL,
                woocommerce_category_id INT NOT NULL UNIQUE,
                PRIMARY KEY (id),
                CONSTRAINT fk_emkp_categories_match FOREIGN KEY (emag_category_id) 
                REFERENCES {$tablenames['EMKP_CATEGORIES']}(emag_category_id) ON DELETE CASCADE
                ) $collate;
                " );
            dbDelta( $sql );

            $sql = ( "
        CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_MATCH_CHARACTERISTICS_ATTRIBUTES']}
                (id mediumint(9) unsigned NOT NULL auto_increment,
                match_id mediumint(9) unsigned NOT NULL,
                emag_characteristic_id INT NOT NULL,
                woocommerce_attribute_id INT NOT NULL,
                PRIMARY KEY (id),
                CONSTRAINT fk_emkp_match_characteristics_attributes FOREIGN KEY (match_id) 
                REFERENCES {$tablenames['EMKP_CATEGORIES_MATCH']}(id) ON DELETE CASCADE
                ) $collate;
                " );
            dbDelta( $sql );

            $sql = "
                CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_VAT_RATES']}
                ( `id` INT(3) NOT NULL AUTO_INCREMENT,
                  `vat_id` INT NOT NULL,
                  `vat_rate` FLOAT NOT NULL,
                  `is_default` BOOLEAN NOT NULL,
                  PRIMARY KEY (`id`),
                  UNIQUE (`vat_id`)
                ) $collate
            ";

            dbDelta( $sql );

            $sql = "
                CREATE TABLE IF NOT EXISTS `{$tablenames['EMKP_PRODUCTS_QUEUE']}` (
                  `id` int(2) NOT NULL AUTO_INCREMENT,
                  `product_id` bigint(20) UNSIGNED NOT NULL,
                  `status` enum('pending','running','sent','failed') NOT NULL DEFAULT 'pending',
                  `state` VARCHAR(255) NOT NULL,
                  `exception_message` TEXT NULL ,
                  `created_at` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
                  `updated_at` datetime NULL ON UPDATE CURRENT_TIMESTAMP,
                  PRIMARY KEY (`id`),
                  KEY `product_id` (`product_id`)
                ) $collate";

            dbDelta( $sql );

            $sql = ( "
        CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_ORDER_IDS']}
                (id mediumint(9) unsigned NOT NULL auto_increment,
                emag_order_id INT NOT NULL,
                wc_order_id INT,
                status int(1) NOT NULL DEFAULT 0,
                created_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                last_updated TIMESTAMP NOT NULL ON UPDATE CURRENT_TIMESTAMP,
                response TEXT,
                PRIMARY KEY (id),
                UNIQUE KEY (emag_order_id)
                ) $collate;
                " );
            dbDelta( $sql );

            $sql = "CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_ORDERS_INVOICES']}
                  (id MEDIUMINT(9) UNSIGNED NOT NULL AUTO_INCREMENT,
                  order_id   INT,
                  file_name  TEXT,
                  url  TEXT,
                  created_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                  PRIMARY KEY (id)
                ) $collate;";
            dbDelta( $sql );

            $sql = ( "
        CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_LOCALITIES']}
                (id mediumint(9) unsigned NOT NULL auto_increment,
                emag_id INT NOT NULL,
                name VARCHAR (60),
                region VARCHAR (60),
                created_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                last_updated TIMESTAMP NOT NULL ON UPDATE CURRENT_TIMESTAMP,
                PRIMARY KEY (id),
                UNIQUE KEY (emag_id)
                ) $collate;
                " );
            dbDelta( $sql );

            $sql = ( "
        CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_COURIERS']}
                (id mediumint(9) unsigned NOT NULL auto_increment,
                emag_id INT NOT NULL,
                name VARCHAR (60),
                display_name VARCHAR (60),
                created_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                last_updated TIMESTAMP NOT NULL ON UPDATE CURRENT_TIMESTAMP,
                PRIMARY KEY (id),
                UNIQUE KEY (emag_id)
                ) $collate;
                " );
            dbDelta( $sql );

            $sql = ( "
        CREATE TABLE IF NOT EXISTS {$tablenames['EMKP_AWBS']}
                (id mediumint(9) unsigned NOT NULL auto_increment,
                emag_id INT NOT NULL,
                order_id bigint(20) unsigned NOT NULL,
                courier_name VARCHAR (60),
                awb_number VARCHAR (60),
                created_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                PRIMARY KEY (id),
                UNIQUE KEY (emag_id),
                CONSTRAINT fk_emkp_awbs_order_id FOREIGN KEY (order_id) 
                REFERENCES {$this->wpdb->prefix}posts(ID) ON DELETE CASCADE
                ) $collate;
                " );

            dbDelta( $sql );

            self::update_db_version();
        }

        /**
         * Method called on plugin delete.
         * Drop tables of this plugin.
         */
        public function uninstall_schema() {
            global $wpdb;

            foreach ( array_reverse( self::$tablenames ) as $tablename ) {
                $wpdb->query( "DROP TABLE IF EXISTS " . $tablename );
            }

            foreach ( self::$config_options as $config_option ) {
                delete_option( $config_option );
            }

            self::update_db_version( false );
        }

        /**
         * Update Emkp version to current.
         *
         * @param null|false|string $version
         */
        public static function update_db_version( $version = null ) {
            delete_option( 'emkp_db_version' );

            if ( $version == false ) {
                return;
            }

            add_option( 'emkp_db_version', is_null( $version ) ? Emkp()->version : $version );
        }
    }

endif;
