<?php
if ( ! class_exists( 'EMKP_Categories_Characteristics_Edit' ) ) :

    /**
     * Class EMKP_Categories_Characteristics_Edit
     */
    class EMKP_Categories_Characteristics_Edit {

        /**
         * @var wpdb
         */
        private $global;

        /**
         * EMKP_Categories constructor.
         */
        public function __construct() {
            global $wpdb;
            $this->global = $wpdb;

            add_action( 'admin_post_nopriv_emkp_save_mapping', function() {exit;} );
            add_action( 'admin_post_emkp_save_mapping', [ $this, 'save_characteristics_attributes' ] );
        }

        /**
         * Initialize admin page
         */
        public function init() {
            add_submenu_page(
                null,
                'Edit Match Characteristics',
                'Edit Match Characteristics',
                'manage_options',
                'emkp-mapping-categories-characteristics-edit',
                [ $this, 'page' ]
            );
        }

        /**
         * Return the view of the page
         */
        public function page() {
            $list = new Emkp_Mapping_Characteristics_List();
            $list->prepare_items();
            $list->views();

            $match_id = (int) $_GET['id'];
            $match    = $this->global->get_row( $this->global->prepare( "SELECT * FROM {$this->global->prefix}emkp_categories_match WHERE id = %d", [ $match_id ] ) );

            $emag_category_name = $this->global->get_row( $this->global->prepare( "SELECT * FROM {$this->global->prefix}emkp_categories WHERE emag_category_id = %d", [ $match->emag_category_id ] ) );

            $woocommerce_category_name = get_term_by( 'id', $match->woocommerce_category_id, 'product_cat', 'ARRAY_A' );
            $woocommerce_category_name = $woocommerce_category_name['name'];
            $mapping_category_edit     = $emag_category_name->name . ' - ' . $woocommerce_category_name;

            // Redirect back to list if mapping does not exist
            if ( ! $match ) {
                wp_redirect( esc_url( admin_url( 'admin.php?page=emkp-mapping-categories' ) ) );
                exit;
            }

            $emag_category        = $match->emag_category_id;
            $woocommerce_category = $match->woocommerce_category_id;

            $emag_category_characteristics   = $this->_get_emag_characteristics( $emag_category );
            $woocommerce_category_attributes = $this->_get_woocommerce_attributes();

            require_once( EMKP_ABSPATH . 'templates/mapping/categories/characteristics/edit.php' );
        }

        /**
         * Save characteristics and attributes
         */
        public function save_characteristics_attributes() {
            $data       = $_POST;
            $match_id = (int) $data['match_id'];
            $table_name = $this->global->prefix . 'emkp_match_characteristics_attributes';

            $woocommerce_categories = [];
            foreach ( $data['woocommerce_categories'] as $item ) {
                if ( filter_var( $item, FILTER_VALIDATE_INT ) || filter_var( $item, FILTER_VALIDATE_INT ) === 0 ) {
                    if ( $item != 0 ) {
                        $woocommerce_categories[] = (int) $item;
                    }
                } else {
                    wp_redirect( esc_url( admin_url( 'admin.php?page=emkp-mapping-categories-characteristics-edit' ) ) . '&id=' . $match_id . '&success=0' );
                    exit;
                }
            }

            if ( ! ( count( array_unique( $woocommerce_categories ) ) === count( $woocommerce_categories ) ) ) {
                wp_redirect( esc_url( admin_url( 'admin.php?page=emkp-mapping-categories-characteristics-edit' ) ) . '&id=' . $match_id . '&success=0' );
                exit;
            }

            $old_data = $this->global->get_results( $this->global->prepare( "SELECT * FROM {$this->global->prefix}emkp_match_characteristics_attributes WHERE match_id = %d", [ $match_id ] ) );

            $old_emag_characteristic_ids = array();
            foreach ( $old_data as $old_item ) {
                $old_emag_characteristic_ids[] = array(
                    'id'                     => $old_item->id,
                    'emag_characteristic_id' => $old_item->emag_characteristic_id
                );
            }

            $not_mapping = [];
            unset( $item );
            foreach ( $data['woocommerce_categories'] as $key => $item ) {

                $key = array_map( 'trim', explode( '_', $key ) );

                if ( ! ( filter_var( $item, FILTER_VALIDATE_INT ) || filter_var( $item, FILTER_VALIDATE_INT ) === 0 )
                     || ! ( filter_var( $key[0], FILTER_VALIDATE_INT ) )
                     || ! ( filter_var( $key[1], FILTER_VALIDATE_INT ) || filter_var( $key[1], FILTER_VALIDATE_INT ) === 0 )
                     || ( $key[1] === '1' && ! ( $item ) )
                ) {
                    wp_redirect( esc_url( admin_url( 'admin.php?page=emkp-mapping-categories-characteristics-edit' ) ) . '&id=' . $match_id . '&success=0' );
                    exit;
                }

                if ( $item ) {
                    $check = $this->global->get_row( $this->global->prepare( "SELECT * FROM $table_name WHERE match_id = $match_id AND emag_characteristic_id = %d", [ $key[0] ] ) );

                    $data_to_save = array(
                        'match_id'                 => $match_id,
                        'emag_characteristic_id'   => $key[0],
                        'woocommerce_attribute_id' => $item
                    );

                    if ( $check ) {
                        unset( $data_to_save['match_id'], $data_to_save['emag_characteristic_id'] );

                        $this->global->update( $table_name, $data_to_save, array( 'ID' => $check->id ) );
                    } else {
                        $this->global->insert( $table_name, $data_to_save );
                    }
                } else {
                    $not_mapping[] = $key[0];
                }
            }

            foreach ( $old_emag_characteristic_ids as $old_emag_characteristic_id ) {
                if ( in_array( $old_emag_characteristic_id['emag_characteristic_id'], $not_mapping ) ) {
                    $this->global->delete( $table_name, array( 'id' => $old_emag_characteristic_id['id'] ) );
                }
            }

            wp_redirect( esc_url( admin_url( 'admin.php?page=emkp-mapping-categories-characteristics-edit' ) ) . '&id=' . $match_id . '&success=1' );
            exit;
        }

        /**
         * Return all of emag characteristics for current category
         *
         * @param $category_id
         *
         * @return array
         */
        public function _get_emag_characteristics( $category_id ) {

            $match_characteristics_attributes = $this->_get_match_characteristics_attributes();

            $categories_characteristics = $this->global->get_results( $this->global->prepare( "SELECT ec.*, ecc.is_mandatory FROM {$this->global->prefix}emkp_categories_characteristics ecc 
              INNER JOIN {$this->global->prefix}emkp_characteristics ec ON ec.emag_characteristic_id = ecc.emag_characteristic_id 
              WHERE ecc.emag_category_id = %d ORDER BY ecc.is_mandatory DESC, ec.name ASC", [ $category_id ] ) );

            $characteristics = [];
            foreach ( $categories_characteristics as $categories_characteristic ) {
                $characteristics[] = array(
                    'characteristic_id'     => $categories_characteristic->id,
                    'name'                  => $categories_characteristic->name . ( $categories_characteristic->is_mandatory ? ' ***' : null ),
                    'is_mandatory'          => $categories_characteristic->is_mandatory,
                    'allow_new_value'       => $categories_characteristic->allow_new_value,
                    'woocommerce_attribute' => array_key_exists( $categories_characteristic->id, $match_characteristics_attributes ) ? $match_characteristics_attributes[ $categories_characteristic->id ] : null
                );
            }

            return $characteristics;
        }

        /**
         * Return all of WooCommerce attributes
         *
         * @return array
         */
        public function _get_woocommerce_attributes() {

            $match_id                      = (int) $_GET['id'];
            $selected_woocommerce_attr_ids = $this->global->get_col( $this->global->prepare( "SELECT woocommerce_attribute_id FROM {$this->global->prefix}emkp_match_characteristics_attributes WHERE match_id = %d", [ $match_id ] ) );

            $attributes = wc_get_attribute_taxonomies();
            $data[]     = array(
                'attribute_id'    => 0,
                'attribute_label' => 'Choose an attribute',
                'disabled'        => 0
            );

            foreach ( $attributes as $attribute ) {

                $data[] = array(
                    'attribute_id'    => $attribute->attribute_id,
                    'attribute_label' => $attribute->attribute_label,
                    'disabled'        => (int) in_array( $attribute->attribute_id, $selected_woocommerce_attr_ids )
                );
            }

            return $data;
        }

        public function _get_match_characteristics_attributes() {
            $match_id = (int) $_GET['id'];
            $result   = $this->global->get_results( $this->global->prepare( "SELECT * FROM {$this->global->prefix}emkp_match_characteristics_attributes WHERE match_id = %d", [ $match_id ] ) );

            $data = array();
            foreach ( $result as $item ) {
                $data[ $item->emag_characteristic_id ] = $item->woocommerce_attribute_id;
            }

            return $data;
        }
    }

    new EMKP_Categories_Characteristics_Edit();

endif;
