<?php
ini_set( 'max_execution_time', 600 );
ini_set( 'max_input_time', 600 );
ini_set( 'memory_limit', '512M' );
ini_set( 'default_socket_timeout', 600 );
set_time_limit( 600 );

if ( ! class_exists( 'EMKP_Categories' ) ) :

    /**
     * Class EMKP_Categories
     */
    class EMKP_Categories {

        /**
         * @var wpdb
         */
        private $global;

        /**
         * EMKP_Categories constructor.
         */
        public function __construct() {
            add_action( 'wp_ajax_nopriv_add_match_event', function() {exit;} );
            add_action( 'wp_ajax_nopriv_import_sync_categories_event', function(){ exit;} );
            add_action( 'admin_post_nopriv_emkp_save_match', function() {exit;} );

            add_action( 'wp_ajax_add_match_event', [ $this, 'add_match_event_callback' ] );
            add_action( 'wp_ajax_import_sync_categories_event', [ $this, 'import_sync_categories_event_callback' ] );
            add_action( 'admin_post_emkp_save_match', [ $this, 'save_categories_match' ] );

            global $wpdb;
            $this->global = $wpdb;
        }

        /**
         * Initialize admin page
         */
        public function init() {
            add_submenu_page( 'emkp_general', 'Mapping Categories', 'Mapping Categories', 'manage_options', 'emkp-mapping-categories', [
                $this,
                'page'
            ] );
        }

        /**
         * Return the view of the page
         */
        public function page() {
            $list = new Emkp_Mapping_List();
            $list->prepare_items();
            $list->views();

            require_once( EMKP_ABSPATH . 'templates/mapping/categories.php' );
        }

        /**
         * Import categories or sync/update if already imported / Ajax Request
         */
        public function import_sync_categories_event_callback() {
            $emkp_api_url  = get_option( 'emkp_api_url' );
            $emkp_username = get_option( 'emkp_username' );
            $emkp_password = get_option( 'emkp_password' );

            if ( empty( $emkp_api_url ) || empty( $emkp_username ) || empty( $emkp_password ) ) {
                wp_send_json( array( 'success' => false ) );

                return null;
            }

            $success = $this->emag_api_request();

            wp_send_json( array( 'success' => $success ) );

            return null;
        }

        /**
         * Send request to Emag Marketplace
         *
         * @return bool
         */
        public function emag_api_request() {
            $emag_data = [];

            $page = 1;
            do {
                $request  = new \Zitec\MkpApiWrapper\Requests\Category\ReadRequest( [
                    'currentPage'  => $page,
                    'is_allowed'   => 1,
                    'itemsPerPage' => 500
                ] );
                $response = \Zitec\MkpApiWrapper\Client::sendRequest( $request );

                if ( ! is_object( $response ) || ! property_exists( $response, 'isError' ) || $response->isError ) {
                    return false;
                }

                $emag_data = array_merge( $emag_data, $response->results );

                $page ++;
            } while ( count( $response->results ) );


            $data = [
                'categories'                 => [],
                'characteristics'            => [],
                'categories_characteristics' => []
            ];

            foreach ( $emag_data as $item ) {
                if ( ! property_exists( $item, 'is_allowed' ) || ! $item->is_allowed ) {
                    continue;
                }

                $data['categories'][ $item->id ] = array(
                    'emag_category_id' => $item->id,
                    'emag_parent_id'   => $item->parent_id,
                    'name'             => $item->name
                );

                foreach ( $item->characteristics as $characteristic ) {
                    $data['characteristics'][ $characteristic->id ] = array(
                        'emag_category_id'       => $item->id,
                        'emag_characteristic_id' => $characteristic->id,
                        'name'                   => $characteristic->name,
                        'type_id'                => $characteristic->type_id,
                        'display_order'          => $characteristic->display_order,
                        'allow_new_value'        => $characteristic->allow_new_value
                    );

                    $data['categories_characteristics'][] = array(
                        'emag_category_id'       => $item->id,
                        'emag_characteristic_id' => $characteristic->id,
                        'is_mandatory'           => $characteristic->is_mandatory,
                    );
                }
            }

            unset( $item, $characteristic, $category );

            $available_emag_categories_ids = [];

            $available_emag_characteristics_ids = [];

            // Prepare insert values for categories
            $categories_insert_values = [];
            foreach ( $data['categories'] as $category ) {
                $available_emag_categories_ids[] = $category['emag_category_id'];
                $emag_parent_id                  = is_null( $category['emag_parent_id'] ) ? 'null' : $category['emag_parent_id'];

                $parameters = [ $category['emag_category_id'], $emag_parent_id, '\'' . $category['name'] . '\'' ];

                $categories_insert_values[] = '(' . implode( ',', $parameters ) . ')';
            }
            $categories_insert_values = implode( ',', $categories_insert_values );


            // Prepare insert values for characteristics
            $characteristics_insert_values = [];
            foreach ( $data['characteristics'] as $characteristic ) {
                $available_emag_characteristics_ids[] = $characteristic['emag_characteristic_id'];

                $parameters = [
                    $characteristic['emag_characteristic_id'],
                    '\'' . $characteristic['name'] . '\'',
                    $characteristic['type_id'],
                    $characteristic['display_order'],
                    $characteristic['allow_new_value']
                ];

                $characteristics_insert_values[] = '(' . implode( ',', $parameters ) . ')';
            }
            $characteristics_insert_values = implode( ',', $characteristics_insert_values );


            // Prepare insert values for categories characteristics
            $categories_characteristics_insert_values = [];
            foreach ( $data['categories_characteristics'] as $categories_characteristic ) {

                $parameters = [
                    $categories_characteristic['emag_category_id'],
                    $categories_characteristic['emag_characteristic_id'],
                    $categories_characteristic['is_mandatory']
                ];

                $key = implode( ',', $parameters );
                if ( ! array_key_exists( $key, $categories_characteristics_insert_values ) ) {
                    $categories_characteristics_insert_values[ $key ] = '(' . implode( ',', $parameters ) . ')';
                }
            }
            $categories_characteristics_insert_values = implode( ',', $categories_characteristics_insert_values );


            // Start transaction
            $this->global->query( 'START TRANSACTION' );

            // Delete unavailable categories
            $deleted_old_categories = true;

            if ( ! empty( $available_emag_categories_ids ) ) {
                $categories_ids       = implode( ',', $available_emag_categories_ids );
                $categoriesOfProducts = $this->global->get_col( "SELECT woocommerce_category_id FROM {$this->global->prefix}emkp_categories_match WHERE emag_category_id NOT IN (" . $categories_ids . ") " );

                if ( is_array( $categoriesOfProducts ) && ! empty( $categoriesOfProducts ) ) {
                    $args = array(
                        'post_type' => 'product',
                        'tax_query' => array(
                            array(
                                'taxonomy' => 'product_cat',
                                'field'    => 'term_id', //This is optional, as it defaults to 'term_id'
                                'terms'    => $categoriesOfProducts,
                                'operator' => 'IN' // Possible values are 'IN', 'NOT IN', 'AND'.
                            )
                        )
                    );

                    do_action( 'emkp_product_category_deleted', query_posts( $args ) );
                }
                $deleted_old_categories = $this->global->query( "DELETE FROM  {$this->global->prefix}emkp_categories WHERE emag_category_id NOT IN (" . $categories_ids . ")" );
                $deleted_old_categories = is_bool( $deleted_old_categories ) ? $deleted_old_categories : true;


            }

            // Delete unavailable characteristics
            $deleted_old_characteristics = true;
            if ( ! empty( $available_emag_characteristics_ids ) ) {
                $deleted_old_characteristics = $this->global->query( "DELETE FROM {$this->global->prefix}emkp_characteristics WHERE emag_characteristic_id NOT IN (" . implode( ',', $available_emag_characteristics_ids ) . ")" );
                $deleted_old_characteristics = is_bool( $deleted_old_characteristics ) ? $deleted_old_characteristics : true;
            }

            $insert_into_emkp_categories = true;
            if ( strlen( $categories_insert_values ) ) {
                $insert_into_emkp_categories = $this->global->query( "INSERT INTO {$this->global->prefix}emkp_categories
            ( emag_category_id, emag_parent_id, name)
            VALUES
            $categories_insert_values
            ON DUPLICATE KEY UPDATE
            emag_parent_id = VALUES(emag_parent_id),
            name = VALUES(name)
            " );
            }

            $insert_into_emkp_characteristics = true;
            if ( strlen( $characteristics_insert_values ) ) {
                $insert_into_emkp_characteristics = $this->global->query( "INSERT INTO {$this->global->prefix}emkp_characteristics
            ( emag_characteristic_id, name, type_id, display_order, allow_new_value)
            VALUES
            $characteristics_insert_values
            ON DUPLICATE KEY UPDATE
            name = VALUES(name),
            type_id = VALUES(type_id),
            display_order = VALUES(display_order),
            allow_new_value = VALUES(allow_new_value)
            " );
            }


            $truncate_emkp_categories_characteristics    = true;
            $insert_into_emkp_categories_characteristics = true;
            if ( ! empty( $categories_characteristics_insert_values ) ) {
                $truncate_emkp_categories_characteristics = $this->global->query( "TRUNCATE TABLE {$this->global->prefix}emkp_categories_characteristics" );
                if ( strlen( $categories_characteristics_insert_values ) ) {
                    $insert_into_emkp_categories_characteristics = $this->global->query( "INSERT INTO {$this->global->prefix}emkp_categories_characteristics
            ( emag_category_id, emag_characteristic_id, is_mandatory)
            VALUES
            $categories_characteristics_insert_values
            " );
                }
            }

            if ( $deleted_old_categories && $deleted_old_characteristics && ! ( $insert_into_emkp_categories === false ) && ! ( $insert_into_emkp_characteristics === false ) && $truncate_emkp_categories_characteristics && ! ( $insert_into_emkp_categories_characteristics === false ) ) {
                $this->global->query( 'COMMIT' ); // if you come here then well done

                return true;
            } else {
                $this->global->query( 'ROLLBACK' ); // // something went wrong, Rollback

                return false;
            }
        }

        /**
         * Save math categories
         */
        public function save_categories_match() {
            unset( $_POST['action'] );

            $mappings = array_key_exists( 'mapping', $_POST ) ? $_POST['mapping'] : [];

            $woocommerce_categories = [];
            $to_delete              = [];
            foreach ( $mappings as $mapping ) {
                if ( ( filter_var( $mapping['emag_category_id'], FILTER_VALIDATE_INT ) || filter_var( $mapping['emag_category_id'], FILTER_VALIDATE_INT ) === 0 ) && ( filter_var( $mapping['woocommerce_category_id'],
                            FILTER_VALIDATE_INT ) || filter_var( $mapping['woocommerce_category_id'], FILTER_VALIDATE_INT ) === 0 )
                ) {

                    if ( $mapping['woocommerce_category_id'] != 0 ) {
                        $woocommerce_categories[] = (int) $mapping['woocommerce_category_id'];
                    }
                } else {
                    $save_categories = 0;
                    wp_redirect( esc_url( admin_url( 'admin.php?page=emkp-mapping-categories' ) ) . '&save_categories=' . $save_categories );
                    exit;
                }

                if ( $mapping['emag_category_id'] == 0 || $mapping['woocommerce_category_id'] == 0 ) {
                    $to_delete[] = (int) $mapping['mapping_id'];
                }
            }

            if ( ! empty( $to_delete ) ) {
                $this->global->query( "DELETE FROM {$this->global->prefix}emkp_categories_match WHERE id IN (" . implode( ',', $to_delete ) . ")" );
            }

            if ( ! ( count( array_unique( $woocommerce_categories ) ) === count( ( $woocommerce_categories ) ) ) ) {
                $save_categories = 0;
                wp_redirect( esc_url( admin_url( 'admin.php?page=emkp-mapping-categories' ) ) . '&save_categories=' . $save_categories );
                exit;
            }

            $deleted_mappings = array_key_exists( 'deleted_mappings', $_POST ) ? $_POST['deleted_mappings'] : '';

            $deleted_mappings = json_decode( $deleted_mappings );

            if ( ! is_array( $mappings ) ) {
                wp_redirect( esc_url( admin_url( 'admin.php?page=emkp-mapping-categories' ) ) );
                exit;
            }

            if ( is_array( $deleted_mappings ) ) {
                $deleted_mappings = array_map( function ( $row ) {
                    return (int) $row;
                }, $deleted_mappings );

                if ( ! empty( $deleted_mappings ) ) {
                    $this->global->query( "DELETE FROM {$this->global->prefix}emkp_categories_match WHERE id IN (" . implode( ',', $deleted_mappings ) . ")" );
                }
            }

            unset( $mapping );
            $to_insert = [];
            foreach ( $mappings as $mapping ) {

                if ( $mapping['emag_category_id'] == 0 || $mapping['woocommerce_category_id'] == 0 ) {
                    continue;
                }

                $parameters  = array(
                    'id'                      => array_key_exists( 'mapping_id', $mapping ) ? $mapping['mapping_id'] : 'NULL',
                    'emag_category_id'        => $mapping['emag_category_id'],
                    'woocommerce_category_id' => $mapping['woocommerce_category_id']
                );
                $to_insert[] = '(' . implode( ',', $parameters ) . ')';
            }

            $to_insert = implode( ',', $to_insert );


            $originalMappings       = $this->global->get_results( "SELECT emag_category_id, woocommerce_category_id FROM {$this->global->prefix}emkp_categories_match" );
            $originalMappingsByWcId = [];
            foreach ( $originalMappings as $mapping ) {
                $originalMappingsByWcId[ $mapping->woocommerce_category_id ] = $mapping->emag_category_id;
            }

            $mappingsToDelete = [];
            foreach ( $mappings as $mapping ) {

                if ( array_key_exists( $mapping['woocommerce_category_id'], $originalMappingsByWcId ) && $mapping['emag_category_id'] !== $originalMappingsByWcId[ $mapping['woocommerce_category_id'] ] ) {
                    $mappingsToDelete[] = $mapping['woocommerce_category_id'];
                }
            }

            if ( ! empty( $mappingsToDelete ) ) {
                $this->global->query( "DELETE FROM {$this->global->prefix}emkp_categories_match WHERE woocommerce_category_id IN (" . implode( ',', $mappingsToDelete ) . ")" );
            }

            $this->global->query( "INSERT INTO {$this->global->prefix}emkp_categories_match
                    (id, emag_category_id, woocommerce_category_id)
                    VALUES
                    $to_insert
                    ON DUPLICATE KEY UPDATE
                    emag_category_id = VALUES(emag_category_id),
                    woocommerce_category_id = VALUES(woocommerce_category_id)
                " );

            $save_categories = true;
            wp_redirect( esc_url( admin_url( 'admin.php?page=emkp-mapping-categories' ) ) . '&save_categories=' . $save_categories );
            exit;
        }

        /**
         * Insert or update data in DB
         *
         * @param       $table
         * @param array $where
         * @param array $data
         */
        public function insert_or_update_match( $table, array $where, array $data ) {
            $tablename = $this->global->prefix . $table;
            $where_key = key( $where );

            $check_data = $this->global->get_row( $this->global->prepare( "SELECT * FROM $tablename WHERE $where_key = %s", [ $where[ $where_key ] ] ) );

            if ( $check_data ) {
                if ( $check_data->emag_category_id != $data['emag_category_id'] ) {
                    $id     = $where['id'];
                    $result = $this->global->get_results( $this->global->prepare( "SELECT * FROM {$this->global->prefix}emkp_match_characteristics_attributes  WHERE match_id = %d", [ $id ] ) );

                    if ( ! empty( $result ) ) {
                        foreach ( $result as $item ) {
                            $match_charactersitics_attributes[] = $item->id;
                        }
                        $match_charactersitics_attributes = implode( ',', $match_charactersitics_attributes );
                        $this->global->query( "DELETE FROM {$this->global->prefix}emkp_match_characteristics_attributes WHERE id IN ($match_charactersitics_attributes)" );
                    }
                }

                $this->global->update( $tablename, $data, array( $where_key => $where[ $where_key ] ) );

            } else {
                $this->global->insert( $tablename, $data );
            }
        }

        /**
         * Ajax callback
         */
        public function add_match_event_callback() {
            $last_id = $this->global->get_col( "SELECT id FROM {$this->global->prefix}emkp_categories_match ORDER BY id DESC LIMIT 1" );
            $last_id = ( count( $last_id ) ? $last_id[0] : 0 ) + (int) $_POST['increment'];

            $list = new Emkp_Mapping_List();
            $list->prepare_items();

            $selected_woocommerce_options = (array) $_POST['selectedWooCommerceOptions'];

            $emag_categories        = $list->emag_categories;
            $woocommerce_categories = $list->woocommerce_categories;

            foreach ( $woocommerce_categories as $key => $woocommerce_category ) {
                if ( in_array( $woocommerce_category['id'], $selected_woocommerce_options ) ) {
                    $woocommerce_categories[ $key ] = array(
                        'id'       => $woocommerce_category['id'],
                        'name'     => $woocommerce_category['name'],
                        'disabled' => 1
                    );
                }
            }

            require_once( EMKP_ABSPATH . 'templates/mapping/categories/partials/ajax/add_new_category.php' );
            exit;
        }
    }

    new EMKP_Categories();

endif;
