<?php

if ( ! class_exists( 'Emkp_Mapping_Characteristics_List' ) ) :
    /**
     * Class Emkp_Mapping_Characteristics_List
     */
    class Emkp_Mapping_Characteristics_List extends Emkp_Wp_List_Table {
        /**
         * @var int
         */
        protected $row = 0;

        /**
         * @var array
         */
        public $emag_characteristic = [];

        /**
         * @var array
         */
        public $woocommerce_attributes = [];

        /**
         * @var int
         */
        protected $last_mapping_id = 0;

        /**
         * @var global
         */
        private $global;

        /**
         * Emkp_Mapping_List constructor.
         */
        public function __construct() {

            global $wpdb;
            $this->global = $wpdb;

            parent::__construct( [
                'plural'   => '',
                'singular' => '',
                'ajax'     => false,
            ] );
        }

        /**
         * Get list columns.
         *
         * @return array
         */
        public function get_columns() {
            return array(
                'emag_characteristics'  => __( 'Mapping Emag Characteristics', 'woocommerce' ),
                'woocommerce_attribute' => __( 'Mapping WooCommerce Attributes', 'woocommerce' ),
            );
        }

        /**
         * @param array $row
         *
         * @return string
         */
        public function column_emag_characteristics( $row ) {

            $required = $row['is_mandatory'] ? 'emkp-required' : '';

            $select = '<input type="text" class="emkp-input ' . $required . '" name="category_characteristic[' . $row['characteristic_id'] . ']" value="' . $row['name'] . '" readonly>';

            return $select;
        }

        /**
         * @param $row
         *
         * @return string
         */
        public function column_woocommerce_attribute( $row ) {

            $required = $row['is_mandatory'] ? 'required="required"' : '';

            $select = '<select class="js-example-basic-single" name="woocommerce_categories[' . $row['characteristic_id'] . "_" . $row['is_mandatory'] . ']"' . $required . '>';

            foreach ( $this->woocommerce_attributes as $woocommerce_attribute ) {

                $selected = $woocommerce_attribute['attribute_id'] == $row['woocommerce_attribute'] ? 'selected ' : null;
                $disabled = $woocommerce_attribute['disabled'] ? 'disabled ' : '';

                if ( ! is_null( $selected ) ) {
                    $disabled = '';
                }

                $select .= '<option value="' . $woocommerce_attribute['attribute_id'] . '" ' . $selected .
                           $disabled . '>' . $woocommerce_attribute['attribute_label'] . '</option>';
            }

            return $select;
        }

        /**
         * Get data for WP_Table
         */
        public function prepare_items() {
            $match_id = (int) $_GET['id'];
            $match    = $this->global->get_row( $this->global->prepare( "SELECT * FROM {$this->global->prefix}emkp_categories_match WHERE id = %d",
                [ $match_id ] ) );

            // Redirect back to list if mapping does not exist
            if ( ! $match ) {
                wp_redirect( esc_url( admin_url( 'admin.php?page=emkp-mapping-categories' ) ) );
                exit;
            }

            $emag_category = $match->emag_category_id;

            $this->_column_headers = [ $this->get_columns(), [], [] ];

            $this->items                  = $this->_get_emag_characteristics( $emag_category );
            $this->woocommerce_attributes = $this->_get_woocommerce_attributes();
        }

        /**
         * Return all of emag characteristics for current category
         *
         * @param $category_id
         *
         * @return array
         */
        public function _get_emag_characteristics( $category_id ) {

            $match_characteristics_attributes = $this->_get_match_characteristics_attributes();

            $categories_characteristics = $this->global->get_results( $this->global->prepare( "SELECT ec.*,ecc.is_mandatory FROM {$this->global->prefix}emkp_categories_characteristics ecc 
              INNER JOIN {$this->global->prefix}emkp_characteristics ec ON ec.emag_characteristic_id = ecc.emag_characteristic_id 
              WHERE ecc.emag_category_id = %d ORDER BY ecc.is_mandatory DESC, ec.name ASC", [ $category_id ] ) );

            $characteristics = [];
            foreach ( $categories_characteristics as $categories_characteristic ) {
                $characteristics[] = array(
                    'characteristic_id'     => $categories_characteristic->id,
                    'name'                  => $categories_characteristic->name . ( $categories_characteristic->is_mandatory ? ' ***' : null ),
                    'is_mandatory'          => $categories_characteristic->is_mandatory,
                    'allow_new_value'       => $categories_characteristic->allow_new_value,
                    'woocommerce_attribute' => array_key_exists( $categories_characteristic->id,
                        $match_characteristics_attributes ) ? $match_characteristics_attributes[ $categories_characteristic->id ] : null
                );
            }

            return $characteristics;
        }

        /**
         * Return all of WooCommerce attributes
         *
         * @return array
         */
        public function _get_woocommerce_attributes() {

            $match_id                      = (int) $_GET['id'];
            $selected_woocommerce_attr_ids = $this->global->get_col( $this->global->prepare( "SELECT woocommerce_attribute_id FROM {$this->global->prefix}emkp_match_characteristics_attributes WHERE match_id = %d",
                [ $match_id ] ) );

            $attributes = wc_get_attribute_taxonomies();
            $data[]     = array(
                'attribute_id'    => 0,
                'attribute_label' => 'Choose an attribute',
                'disabled'        => 0
            );

            foreach ( $attributes as $attribute ) {

                $data[] = array(
                    'attribute_id'    => $attribute->attribute_id,
                    'attribute_label' => $attribute->attribute_label,
                    'disabled'        => (int) in_array( $attribute->attribute_id, $selected_woocommerce_attr_ids )
                );
            }

            return $data;
        }

        /**
         * Get Characteristics and Attributes for Mapped Category
         *
         * @return array
         */
        public function _get_match_characteristics_attributes() {
            $match_id = (int) $_GET['id'];
            $result   = $this->global->get_results( $this->global->prepare( "SELECT * FROM {$this->global->prefix}emkp_match_characteristics_attributes WHERE match_id = %d",
                [ $match_id ] ) );

            $data = array();
            foreach ( $result as $item ) {
                $data[ $item->emag_characteristic_id ] = $item->woocommerce_attribute_id;
            }

            return $data;
        }
    }
endif;
