<?php

if ( ! class_exists( 'Emkp_Mapping_List' ) ) :
    /**
     * Class Emkp_Mapping_List
     */
    class Emkp_Mapping_List extends Emkp_Wp_List_Table {

        /**
         * @var int
         */
        protected $row = 0;

        /**
         * @var array
         */
        public $emag_categories = [];

        /**
         * @var array
         */
        public $woocommerce_categories = [];

        /**
         * @var int
         */
        protected $last_mapping_id = 0;

        /**
         * Emkp_Mapping_List constructor.
         */
        public function __construct() {
            parent::__construct( [
                'plural'   => '',
                'singular' => '',
                'ajax'     => false,
            ] );
        }

        /**
         * Get list columns.
         *
         * @return array
         */
        public function get_columns() {
            return array(
                'emag_category'        => __( 'Emag Category', 'emkp' ),
                'woocommerce_category' => __( 'WooCommerce Category', 'emkp' ),
                'actions'              => __( 'Actions', 'emkp' ),
            );
        }

        /**
         * @param array $row
         *
         * @return string
         */
        public function column_emag_category( $row ) {
            $select = '<input type="hidden" name="mapping[' . $this->row . '][mapping_id]" value="' . $row['mapping_id'] . '" />';

            $select .= '<select class="js-mapping-emag-category-select" name="mapping[' . $this->row . '][emag_category_id]">';
            $select .= '<option value="0">' . _( 'Choose one Emag Category' ) . '</option>';
            foreach ( $this->emag_categories as $emag_category ) {

                $selected = $emag_category['emag_category_id'] == $row['emag_category_id'] ? 'selected ' : null;

                $select .= '<option value="' . $emag_category['emag_category_id'] . '" ' . $selected . ' >';
                $select .= $emag_category['name'];
                $select .= '</option>';
            }
            $select .= '</select>';

            return $select;
        }

        /**
         * @param array $row
         *
         * @return string
         */
        public function column_woocommerce_category( $row ) {
            $select = '<select class="js-mapping-woocommerce-category-select" name="mapping[' . $this->row . '][woocommerce_category_id]">';
            $select .= '<option value="0">' . _( 'Choose one WooCommerce Category' ) . '</option>';
            foreach ( $this->woocommerce_categories as $woocommerce_category ) {

                $selected = $woocommerce_category['id'] == $row['woocommerce_category_id'] ? 'selected ' : null;
                $disabled = $woocommerce_category['disabled'] ? 'disabled ' : '';

                if ( ! is_null( $selected ) ) {
                    $disabled = '';
                }

                $select .= '<option value="' . $woocommerce_category['id'] . '" ' . $selected . $disabled . ' >';
                $select .= $woocommerce_category['name'];
                $select .= '</option>';
            }
            $select .= '</select>';

            return $select;
        }

        /**
         * @param array $row
         *
         * @return string
         */
        public function column_actions( $row ) {
            if ( $row['mapping_id'] != $this->last_mapping_id ) {
                $this->last_mapping_id = $row['mapping_id'];
                $this->row ++;
            }

            $actions   = [];
            $actions[] = '<a href="' . admin_url( 'admin.php?page=emkp-mapping-categories-characteristics-edit&id=' . $row['mapping_id'] ) . '" class="button">' . __( 'Edit characteristics' ) . '</a>';
            $actions[] = '<button type="button" data-trigger="delete-mapping" data-mapping-id="' . $row['mapping_id'] . '" class="button emkp-margin-left-10">' . __( 'Delete' ) . '</button>';

            return implode( '', $actions );
        }

        /**
         * @return array
         */
        protected function get_bulk_actions() {
            return [];
        }

        /**
         *
         */
        public function prepare_items() {
            global $wpdb;

            $data = $wpdb->get_results( "
            SELECT wecm.id AS mapping_id, wec.emag_category_id, wec.name AS emag_category_name, wt.term_id AS woocommerce_category_id, wt.name AS woocommerce_category_name
            FROM {$wpdb->prefix}emkp_categories_match wecm
            INNER JOIN {$wpdb->prefix}emkp_categories wec ON wec.emag_category_id = wecm.emag_category_id
            INNER JOIN {$wpdb->prefix}term_taxonomy wtt ON wtt.term_id = woocommerce_category_id
            INNER JOIN {$wpdb->prefix}terms wt ON wt.term_id = wtt.term_id;
            ", ARRAY_A );

            $perPage     = 15;
            $currentPage = $this->get_pagenum();
            $totalItems  = count( $data );
            $this->set_pagination_args( array(
                'total_items' => $totalItems,
                'per_page'    => $perPage
            ) );
            $data = array_slice( $data, ( ( $currentPage - 1 ) * $perPage ), $perPage );

            $this->items = $data;

            $this->_column_headers = [ $this->get_columns(), [], [] ];

            $this->emag_categories = $this->get_emag_categories();

            $this->get_woocommerce_categories( $this->woocommerce_categories );

            $mached_woocommerce_categories = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}emkp_categories_match", ARRAY_A );

            $mached_woocommerce_categories_data = [];
            foreach ( $mached_woocommerce_categories as $mached_woocommerce_category ) {
                $mached_woocommerce_categories_data[] = $mached_woocommerce_category['woocommerce_category_id'];
            }

            $woocommerce_categories = [];
            foreach ( $this->woocommerce_categories as $woocommerce_category ) {
                $woocommerce_categories[] = array(
                    'id'       => $woocommerce_category['id'],
                    'name'     => $woocommerce_category['name'],
                    'disabled' => (int) in_array( $woocommerce_category['id'], $mached_woocommerce_categories_data )
                );
            }

            $this->woocommerce_categories = $woocommerce_categories;
        }

        /**
         * @param array $woocommerce_categories
         * @param int $page
         * @param null|int $parent_id
         */
        public function get_woocommerce_categories( &$woocommerce_categories, $page = 0, $parent_id = null ) {
            $all_categories = get_categories( array(
                'taxonomy'         => 'product_cat',
                'orderby'          => 'name',
                'hierarchical'     => 1,
                'hide_empty'       => 0,
                'parent'           => $parent_id,
                'show_option_none' => '',
            ) );

            foreach ( $all_categories as $cat ) {
                $woocommerce_categories[] = [
                    'id'   => $cat->cat_ID,
                    'name' => str_repeat( '- ', $page ) . ( $page ? ' ' : null ) . $cat->name
                ];
                $this->get_woocommerce_categories( $woocommerce_categories, $page + 1, $cat->cat_ID );
            }
        }

        /**
         * @return array
         */
        public function get_emag_categories() {
            global $wpdb;

            $emag_categories         = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}emkp_categories", ARRAY_A );
            $categories_match        = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}emkp_categories_match", ARRAY_A );
            $emag_category_match_ids = [];
            foreach ( $categories_match as $category_match ) {
                $emag_category_match_ids[] = $category_match['emag_category_id'];
            }

            $data = [];
            foreach ( $emag_categories as $emag_category ) {
                $data[] = array(
                    'emag_category_id' => $emag_category['emag_category_id'],
                    'name'             => $emag_category['name'],
                    'disabled'         => (int) in_array( $emag_category['emag_category_id'], $emag_category_match_ids )
                );
            }

            return $data;
        }
    }
endif;
