<?php

namespace Zitec\MkpApiWrapper;

use Zitec\MkpApiWrapper\AlertManager\NullManager;
use Zitec\MkpApiWrapper\AlertManager\AlertManagerInterface;
use Zitec\MkpApiWrapper\Logger\FileLogger;
use Zitec\MkpApiWrapper\Logger\LoggerInterface;
use Zitec\MkpApiWrapper\Requests\AbstractRequest;

/**
 * Client for Emag Marketplace API.
 *
 * Class Client
 * @package Zitec\MkpApiWrapper
 */
final class Client
{
    /**
     * @var InstantiableClient
     */
    private static $instance = null;

    private function __construct()
    {
    }

    /**
     * @param string                $mkp_api_url
     * @param string                $mkp_username
     * @param string                $mkp_password
     * @param LoggerInterface       $logger
     * @param AlertManagerInterface $alertManager
     *
     * @return InstantiableClient
     */
    public static function init(
        $mkp_api_url,
        $mkp_username,
        $mkp_password,
        LoggerInterface $logger = null,
        AlertManagerInterface $alertManager = null
    ) {
        // Check if instance is already exists
        if (self::$instance == null) {
            self::$instance = new InstantiableClient(
                $mkp_api_url,
                $mkp_username,
                $mkp_password,
                $logger,
                $alertManager);
        }

        return self::$instance;
    }

    /**
     * @param AbstractRequest $request
     *
     * @return array
     */
    public static function sendRequest(AbstractRequest $request)
    {
        return self::$instance->sendRequest($request);
    }

    private function __clone()
    {
    }

    private function __wakeup()
    {
    }
}
