<?php

namespace Zitec\MkpApiWrapper\Requests\ProductOffer;

/**
 * Request for updating a product offer on Emag Marketplace.
 *
 * The Update Request for a Product Offer
 * accepts the same parameters as the Create Request
 * but its fields have different constraints.
 *
 * The API Endpoint is the same as the parents' one.
 *
 * Class UpdateRequest
 * @package Zitec\MkpApiWrapper\Requests\ProductOffer
 */
class UpdateRequest extends CreateRequest
{
    /**
     * Override the constraints of the base
     * class (CreateRequest) with the ones of this one.
     *
     * @return array
     */
    public function getFieldConstraints()
    {
        // Extra required fields
        $requiredFields = [
            'id',
            'status',
            'sale_price',
            'vat_id',
            'comission',
            'availability_id',
            'handling_time.',
            'stock.'
        ];

        foreach ($this->_fieldConstraints as $field => $constraints) {
            $firstConstraint = $constraints[0];

            // If the first constraint is an array, skip this field
            if (is_array($firstConstraint)) {
                continue;
            }

            // Get the name of the field up until the first dot (including it)
            $explodedField = explode('.', $field);
            $untilFirstDot = array_shift($explodedField) . '.';

            // Determine the new constraint of the field: optional or required
            $newConstraint = !in_array($field, $requiredFields) && !in_array($untilFirstDot,
                $requiredFields) ? 'optional' : 'required';

            // Set the new constraint
            if ($firstConstraint != $newConstraint) {
                $this->_fieldConstraints[$field][0] = $newConstraint;
            }
        }

        // Add new constraints
        $this->_fieldConstraints['commission'] = ['required'];
        $this->_fieldConstraints['availability_id'] = $this->_fieldConstraints['availability.*.id'];

        return $this->_fieldConstraints;
    }
}
